@@ -35,6 +35,7 @@
 #include "kleopatra_debug.h"
 
 #include "certificatelineedit.h"
+#include "unknownrecipientwidget.h"
 
 #include <QVBoxLayout>
 #include <QHBoxLayout>
@@ -63,28 +64,40 @@ namespace {
 class SignCertificateFilter: public DefaultKeyFilter
 {
 public:
-    SignCertificateFilter() : DefaultKeyFilter()
+    SignCertificateFilter(GpgME::Protocol proto) : DefaultKeyFilter()
     {
         setRevoked(DefaultKeyFilter::NotSet);
         setExpired(DefaultKeyFilter::NotSet);
         setHasSecret(DefaultKeyFilter::Set);
         setCanSign(DefaultKeyFilter::Set);
+
+        if (proto == GpgME::OpenPGP) {
+            setIsOpenPGP(DefaultKeyFilter::Set);
+        } else if (proto == GpgME::CMS) {
+            setIsOpenPGP(DefaultKeyFilter::NotSet);
+        }
     }
 };
 class EncryptCertificateFilter: public DefaultKeyFilter
 {
 public:
-    EncryptCertificateFilter(): DefaultKeyFilter()
+    EncryptCertificateFilter(GpgME::Protocol proto): DefaultKeyFilter()
     {
         setRevoked(DefaultKeyFilter::NotSet);
         setExpired(DefaultKeyFilter::NotSet);
         setCanEncrypt(DefaultKeyFilter::Set);
+
+        if (proto == GpgME::OpenPGP) {
+            setIsOpenPGP(DefaultKeyFilter::Set);
+        } else if (proto == GpgME::CMS) {
+            setIsOpenPGP(DefaultKeyFilter::NotSet);
+        }
     }
 };
 class EncryptSelfCertificateFilter: public EncryptCertificateFilter
 {
 public:
-    EncryptSelfCertificateFilter(): EncryptCertificateFilter()
+    EncryptSelfCertificateFilter(GpgME::Protocol proto): EncryptCertificateFilter(proto)
     {
         setRevoked(DefaultKeyFilter::NotSet);
         setExpired(DefaultKeyFilter::NotSet);
@@ -94,10 +107,11 @@ public:
 };
 }
 
-SignEncryptWidget::SignEncryptWidget(QWidget *parent)
+SignEncryptWidget::SignEncryptWidget(QWidget *parent, bool sigEncExclusive)
     : QWidget(parent),
       mModel(AbstractKeyListModel::createFlatKeyListModel(this)),
-      mRecpRowCount(2)
+      mRecpRowCount(2),
+      mIsExclusive(sigEncExclusive)
 {
     QVBoxLayout *lay = new QVBoxLayout(this);
     lay->setMargin(0);
@@ -107,19 +121,18 @@ SignEncryptWidget::SignEncryptWidget(QWidget *parent)
     /* The signature selection */
     QHBoxLayout *sigLay = new QHBoxLayout;
     QGroupBox *sigGrp = new QGroupBox(i18n("Prove authenticity (sign)"));
-    QCheckBox *sigChk = new QCheckBox(i18n("Sign as:"));
-    sigChk->setChecked(true);
+    mSigChk = new QCheckBox(i18n("Sign as:"));
+    mSigChk->setChecked(true);
 
     mSigSelect = new KeySelectionCombo();
-    mSigSelect->setKeyFilter(std::shared_ptr<KeyFilter>(new SignCertificateFilter()));
 
-    sigLay->addWidget(sigChk);
+    sigLay->addWidget(mSigChk);
     sigLay->addWidget(mSigSelect, 1);
     sigGrp->setLayout(sigLay);
     lay->addWidget(sigGrp);
 
-    connect(sigChk, &QCheckBox::toggled, mSigSelect, &QWidget::setEnabled);
-    connect(sigChk, &QCheckBox::toggled, this, &SignEncryptWidget::updateOp);
+    connect(mSigChk, &QCheckBox::toggled, mSigSelect, &QWidget::setEnabled);
+    connect(mSigChk, &QCheckBox::toggled, this, &SignEncryptWidget::updateOp);
     connect(mSigSelect, &KeySelectionCombo::currentKeyChanged,
             this, &SignEncryptWidget::updateOp);
 
@@ -133,17 +146,16 @@ SignEncryptWidget::SignEncryptWidget(QWidget *parent)
 
     // Own key
     mSelfSelect = new KeySelectionCombo();
-    mSelfSelect->setKeyFilter(std::shared_ptr<KeyFilter>(new EncryptSelfCertificateFilter()));
-    QCheckBox *encSelfChk = new QCheckBox(i18n("Encrypt for me:"));
-    encSelfChk->setChecked(true);
-    mRecpLayout->addWidget(encSelfChk, 0, 0);
+    mEncSelfChk = new QCheckBox(i18n("Encrypt for me:"));
+    mEncSelfChk->setChecked(true);
+    mRecpLayout->addWidget(mEncSelfChk, 0, 0);
     mRecpLayout->addWidget(mSelfSelect, 0, 1);
 
     // Checkbox for other keys
-    QCheckBox *encOtherChk = new QCheckBox(i18n("Encrypt for others:"));
-    mRecpLayout->addWidget(encOtherChk, 1, 0);
-    encOtherChk->setChecked(true);
-    connect(encOtherChk, &QCheckBox::toggled, this,
+    mEncOtherChk = new QCheckBox(i18n("Encrypt for others:"));
+    mRecpLayout->addWidget(mEncOtherChk, 1, 0);
+    mEncOtherChk->setChecked(true);
+    connect(mEncOtherChk, &QCheckBox::toggled, this,
         [this](bool toggled) {
             Q_FOREACH (CertificateLineEdit *edit, mRecpWidgets) {
                 edit->setEnabled(toggled);
@@ -179,18 +191,47 @@ SignEncryptWidget::SignEncryptWidget(QWidget *parent)
     // Connect it
     connect(encBox, &QGroupBox::toggled, recipientWidget, &QWidget::setEnabled);
     connect(encBox, &QGroupBox::toggled, this, &SignEncryptWidget::updateOp);
-    connect(encSelfChk, &QCheckBox::toggled, mSelfSelect, &QWidget::setEnabled);
-    connect(encSelfChk, &QCheckBox::toggled, this, &SignEncryptWidget::updateOp);
+    connect(mEncSelfChk, &QCheckBox::toggled, mSelfSelect, &QWidget::setEnabled);
+    connect(mEncSelfChk, &QCheckBox::toggled, this, &SignEncryptWidget::updateOp);
     connect(mSymmetric, &QCheckBox::toggled, this, &SignEncryptWidget::updateOp);
     connect(mSelfSelect, &KeySelectionCombo::currentKeyChanged,
             this, &SignEncryptWidget::updateOp);
 
-    // Ensure that the sigChk is aligned togehter with the encryption check boxes.
-    sigChk->setMinimumWidth(qMax(encOtherChk->width(), encSelfChk->width()));
+    if (mIsExclusive) {
+        connect(mEncOtherChk, &QCheckBox::toggled, this, [this](bool value) {
+            if (mCurrentProto != GpgME::CMS) {
+                return;
+            }
+            if (value) {
+                mSigChk->setChecked(false);
+            }
+        });
+        connect(mEncSelfChk, &QCheckBox::toggled, this, [this](bool value) {
+            if (mCurrentProto != GpgME::CMS) {
+                return;
+            }
+            if (value) {
+                mSigChk->setChecked(false);
+            }
+        });
+        connect(mSigChk, &QCheckBox::toggled, this, [this](bool value) {
+            if (mCurrentProto != GpgME::CMS) {
+                return;
+            }
+            if (value) {
+                mEncSelfChk->setChecked(false);
+                mEncOtherChk->setChecked(false);
+            }
+        });
+    }
+
+    // Ensure that the mSigChk is aligned togehter with the encryption check boxes.
+    mSigChk->setMinimumWidth(qMax(mEncOtherChk->width(), mEncSelfChk->width()));
 
     lay->addWidget(encBox);
 
     loadKeys();
+    setProtocol(GpgME::UnknownProtocol);
     addRecipient(Key());
     updateOp();
 }
@@ -203,7 +244,7 @@ void SignEncryptWidget::addRecipient()
 void SignEncryptWidget::addRecipient(const Key &key)
 {
     CertificateLineEdit *certSel = new CertificateLineEdit(mModel, this,
-                                                           new EncryptCertificateFilter());
+                                                           new EncryptCertificateFilter(mCurrentProto));
     mRecpWidgets << certSel;
 
     if (!mRecpLayout->itemAtPosition(mRecpRowCount - 1, 1)) {
@@ -225,9 +266,60 @@ void SignEncryptWidget::addRecipient(const Key &key)
 
     if (!key.isNull()) {
         certSel->setKey(key);
+        mAddedKeys << key;
     }
 }
 
+void SignEncryptWidget::clearAddedRecipients()
+{
+    for (auto w: mUnknownWidgets) {
+        mRecpLayout->removeWidget(w);
+        delete w;
+    }
+
+    for (auto &key: mAddedKeys) {
+        removeRecipient(key);
+    }
+}
+
+void SignEncryptWidget::addUnknownRecipient(const char *keyID)
+{
+    auto unknownWidget = new UnknownRecipientWidget(keyID);
+    mUnknownWidgets << unknownWidget;
+
+    if (!mRecpLayout->itemAtPosition(mRecpRowCount - 1, 1)) {
+        // First widget. Should align with the row above that
+        // contians the encrypt for others checkbox.
+        mRecpLayout->addWidget(unknownWidget, mRecpRowCount - 1, 1);
+    } else {
+        mRecpLayout->addWidget(unknownWidget, mRecpRowCount++, 1);
+    }
+
+    connect(KeyCache::instance().get(), &Kleo::KeyCache::keysMayHaveChanged,
+            this, [this] () {
+        // Check if any unknown recipient can now be found.
+        for (auto w: mUnknownWidgets) {
+            auto key = KeyCache::instance()->findByKeyIDOrFingerprint(w->keyID().toLatin1().constData());
+            if (key.isNull()) {
+                std::vector<std::string> subids;
+                subids.push_back(std::string(w->keyID().toLatin1().constData()));
+                for (const auto &subkey: KeyCache::instance()->findSubkeysByKeyID(subids)) {
+                    key = subkey.parent();
+                }
+            }
+            if (key.isNull()) {
+                continue;
+            }
+            // Key is now available replace by line edit.
+            qCDebug(KLEOPATRA_LOG) << "Removing widget for keyid: " << w->keyID();
+            mRecpLayout->removeWidget(w);
+            mUnknownWidgets.removeAll(w);
+            delete w;
+            addRecipient(key);
+        }
+    });
+}
+
 void SignEncryptWidget::recipientsChanged()
 {
     bool oneEmpty = false;
@@ -358,6 +450,23 @@ void SignEncryptWidget::recpRemovalRequested(CertificateLineEdit *w)
     }
 }
 
+void SignEncryptWidget::removeRecipient(const GpgME::Key &key)
+{
+    for (CertificateLineEdit *edit: mRecpWidgets) {
+        const auto editKey = edit->key();
+        if (key.isNull() && editKey.isNull()) {
+            recpRemovalRequested(edit);
+            return;
+        }
+        if (editKey.primaryFingerprint() &&
+            key.primaryFingerprint() &&
+            !strcmp(editKey.primaryFingerprint(), key.primaryFingerprint())) {
+            recpRemovalRequested(edit);
+            return;
+        }
+    }
+}
+
 bool SignEncryptWidget::encryptSymmetric() const
 {
     return mSymmetric->isChecked();
@@ -383,3 +492,39 @@ void SignEncryptWidget::saveOwnKeys() const
         keys.writeEntry("EncryptKey", encKey.primaryFingerprint());
     }
 }
+
+void SignEncryptWidget::setSigningChecked(bool value)
+{
+    mSigChk->setChecked(value);
+}
+
+void SignEncryptWidget::setEncryptionChecked(bool value)
+{
+    mEncSelfChk->setChecked(value);
+    mEncOtherChk->setChecked(value);
+}
+
+void SignEncryptWidget::setProtocol(GpgME::Protocol proto)
+{
+    if (mCurrentProto == proto) {
+        return;
+    }
+    mCurrentProto = proto;
+    mSigSelect->setKeyFilter(std::shared_ptr<KeyFilter>(new SignCertificateFilter(proto)));
+    mSelfSelect->setKeyFilter(std::shared_ptr<KeyFilter>(new EncryptSelfCertificateFilter(proto)));
+    const auto encFilter = std::shared_ptr<KeyFilter>(new EncryptCertificateFilter(proto));
+    Q_FOREACH (CertificateLineEdit *edit, mRecpWidgets) {
+        edit->setKeyFilter(encFilter);
+    }
+
+    if (mIsExclusive) {
+        mSymmetric->setDisabled(proto == GpgME::CMS);
+        if (mSymmetric->isChecked() && proto == GpgME::CMS) {
+            mSymmetric->setChecked(false);
+        }
+        if (mSigChk->isChecked() && proto == GpgME::CMS &&
+                (mEncSelfChk->isChecked() || mEncOtherChk->isChecked())) {
+            mSigChk->setChecked(false);
+        }
+    }
+}
